// SIGMA SERVER-SIDE EMBED API - Parameterized Queries QuickStart

// 1: Require necessary Node.js modules
const express = require('express');
const crypto = require('crypto');

// 2: Initialize an Express application
const app = express();

// 3: Manually set your configuration variables here (example values shown)
const EMBED_PATH = 'YOUR_EMBED_PATH';
const EMBED_SECRET = 'YOUR_EMBED_SECRET';
const CLIENT_ID = 'YOUR_EMBED_CLIENTID';
const PORT = 3000; // Feel free to change the port number as needed

// 4: Server Setup
app.get('/', (req, res) => {
    res.sendFile(`${__dirname}/index.html`); // Serve the main HTML file for the root path
});

// 5: Define a route handler for generating Sigma embed URLs
app.get('/api/generate-embed-url', (req, res) => {
    try {
        // Generate a unique nonce using crypto's UUID
        const nonce = crypto.randomUUID();
        let searchParams = `?:nonce=${nonce}`;

        // 6: Construct required search parameters
        searchParams += `&:client_id=${CLIENT_ID}`;

	// CLIENT_A USER CONFIGURATION
	        searchParams += '&:email=user1@client-A.com';
        	searchParams += '&:external_user_id=user1@client-A.com';
        	searchParams += '&:external_user_team=Client_A';
        	searchParams += '&:account_type=Viewer';
		searchParams += '&:ua_Client_DB=CLIENT_A_DB';
	  	searchParams += '&:ua_Client_Schema=Client_A_Schema';

	// CLIENT_B USER CONFIGURATION
        	 // searchParams += '&:email=user1@client-B.com';
        	 // searchParams += '&:external_user_id=user1@client-B.com';
        	 // searchParams += '&:external_user_team=Client_B';
        	 // searchParams += '&:account_type=Viewer';
	  	 // searchParams += '&:ua_Client_DB=CLIENT_B_DB';
	  	 // searchParams += '&:ua_Client_Schema=Client_B_Schema';

        searchParams += '&:mode=userbacked';
        searchParams += '&:session_length=600';
        searchParams += `&:time=${Math.floor(new Date().getTime() / 1000)}`;

        // 8: Construct the URL with search parameters and generate a signature
        const URL_WITH_SEARCH_PARAMS = EMBED_PATH + searchParams;
        const SIGNATURE = crypto
            .createHmac('sha256', Buffer.from(EMBED_SECRET, 'utf8'))
            .update(Buffer.from(URL_WITH_SEARCH_PARAMS, 'utf8'))
            .digest('hex');
        const URL_TO_SEND = `${URL_WITH_SEARCH_PARAMS}&:signature=${SIGNATURE}`;

        // 9: Send the final URL to the requester
        res.status(200).json({ url: URL_TO_SEND });
    } catch (error) {
        console.error('Error generating embed URL:', error.message);
        res.status(500).send('Internal Server Error');
    }
});

// 10: Start the server
app.listen(PORT, () => {
    console.log(`Server listening on port ${PORT}`);
});



